/* Flex declarations */

%option noyywrap
%option debug
%option caseless
%option 8bit
%option never-interactive

%{
/* C declarations */

#include "qb.tab.h"

void yyerror(const char *msg);
void yyerror(const char *msg)
{
    fprintf(stderr, "Hmm: '%s'\n", msg);
}

%}
/* Flex declarations */

digit [0-9]
blank [ \t]

%%
	/* Grammar begins: */

	/* Comments */
REM\ [^:\n]* /* Skip */
\'[^\n]*     /* Skip */

	/* Logical operators */
IMP        return IMP;
EQV        return EQV;
XOR        return XOR;
OR         return  OR;
AND        return AND;
NOT        return NOT;

MOD        return MOD;

	/* If stuff */
IF         return IF;
THEN       return THEN;
ELSE       return ELSE;
ELSEIF     return ELSEIF;
"END IF"   return ENDIF;

	/* Switch stuff */
SELECT     return SELECT;
CASE       return CASE;
"END SELECT"  return ENDSELECT;
IS         return IS;

	/* Loop stuff */
"DO WHILE" return DOWHILE;
"DO UNTIL" return DOUNTIL;
"LOOP WHILE" return LOOPWHILE;
"LOOP UNTIL" return LOOPUNTIL;
WEND       return WEND;
DO         return DO;
LOOP       return LOOP;
UNTIL      return UNTIL;
WHILE      return WHILE;
"EXIT DO"  return EXITDO;

	/* Sub stuff */
SUB           return SUB;
"END SUB"     return ENDSUB;
"EXIT SUB"    return EXITSUB;
"DECLARE SUB" return DECLARESUB;
FUNCTION           return FUNCTION;
"END FUNCTION"     return ENDFUNCTION;
"EXIT FUNCTION"    return EXITFUNCTION;
"DECLARE FUNCTION" return DECLAREFUNCTION;

	/* For stuff */
FOR        return FOR;
TO         return TO;
STEP       return STEP;
NEXT       return NEXT;
"EXIT FOR" return EXITFOR;

	/* Typedef stuff */
DEFINT     return DEFINT;
DEFSNG     return DEFSNG;
DEFDBL     return DEFDBL;
DEFSTR     return DEFSTR;
DEFLNG     return DEFLNG;

INTEGER    return INTEGER;
SINGLE     return SINGLE;
DOUBLE     return DOUBLE;
STRING     return STRING;
LONG       return LONG;
AS         return AS;
TYPE       return TYPE;
"END TYPE" return ENDTYPE;
SHARED     return SHARED;
DIM        return DIM;
CONST      return CONST;

	/* Other */
GOTO       return GOTO;
GOSUB      return GOSUB;
RETURN     return RETURN;
CALL       return CALL;

END        return END;
LET        return LET;

PRINT      return PRINT;
USING      return USING;
LPRINT     return LPRINT;

[a-z][a-z0-9]*[!#%$&]? {
    fprintf(stderr, "Identifier: '%s' (%u)\n", yytext, yyleng);
    return IDENTIFIER;
  }

-?{digit}+(\.{digit}*)?(e[+-]?{digit}+)? {
    fprintf(stderr, "Float: '%s'\n", yytext);
    return NUMBER;
  }

#{digit}* {
    return FILENUM;
  }

\r /* skip \r */

:       return COLONCHAR;
\n      return CRLF;

"<="    return LESSOREQUAL;
">="    return LESSOREQUAL;
"<>"    return NOTEQUAL;
"><"    return NOTEQUAL;
"!="    return NOTEQUAL;

\"[^"\n]*\"  {
    fprintf(stderr, "String: '%s'\n", yytext);
    return STRING;
  }

[-<>=+*/\\,;()] { return yytext[0]; }

{blank}+    /* blank   */

. fprintf(stderr, "Unrecognized char: '%s'\n", yytext);

%%
